---------------------------------------------------------------------------------
--
-- Prat - A framework for World of Warcraft chat mods
--
-- Copyright (C) 2006-2007  Prat Development Team
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to:
--
-- Free Software Foundation, Inc., 
-- 51 Franklin Street, Fifth Floor, 
-- Boston, MA  02110-1301, USA.
--
--
-------------------------------------------------------------------------------



--[[
Name: PratSubstitutions
Revision: $Revision: 28060 $
Author(s): Sylvanaar
Website: http://www.wowace.com/files/index.php?path=Prat/
Documentation: http://www.wowace.com/wiki/Prat/Integrated_Modules#Substitutions
Subversion: http://svn.wowace.com/wowace/trunk/Prat/
Discussions: http://groups.google.com/group/wow-prat
Issues and feature requests: http://code.google.com/p/prat/issues/list
Description: A module to provide basic chat substitutions. (default=off).
Dependencies: Prat
]]

-- set prat module name
local PRAT_MODULE = Prat:RequestModuleName("PratSubstitutions")

-- define localized strings
local L = AceLibrary("AceLocale-2.2"):new(PRAT_MODULE)

L:RegisterTranslations("enUS", function() return {
	["Substitutions"] = true,
	["A module to provide basic chat substitutions."] = true,
    ["leavejoin_name"] = "Filter Channel Leave/Join",
    ["leavejoin_desc"] = "Filter out channel leave/join spam",
} end)


-- get prat module categories
local cat = Prat.Categories

-- create prat module
Prat_Substitutions = Prat:NewModule(PRAT_MODULE)
Prat_Substitutions.revision = tonumber(string.sub("$Revision: 28060 $", 12, -3))

-- define key module values
Prat_Substitutions.moduleName = L["Substitutions"] 	
Prat_Substitutions.moduleDesc = L["A module to provide basic chat substitutions."] 	

Prat_Substitutions.consoleName = string.lower(Prat_Substitutions.moduleName)
Prat_Substitutions.guiName = Prat_Substitutions.moduleName

Prat_Substitutions.Categories	= { cat.FILTER }	
				-- list here: http://groups-beta.google.com/group/wow-prat/web/prat-categories

-- init moduleOoptions (which gets populated by the other
-- various options tables below
Prat_Substitutions.moduleOptions = {}

-- default values for any settings that need them
Prat_Substitutions.defaultDB = {
	on	= false,
}

-- any boolean options
Prat_Substitutions.toggleOptions = {
}


--[[------------------------------------------------
    Module Event Functions
------------------------------------------------]]--

-- things to do when the module is enabled
function Prat_Substitutions:OnModuleEnable()
    -- This will most likely be put into the base code
    for _,v in pairs(Prat_Substitutions.modulePatterns) do
        Prat:RegisterPattern(v, self.name)
    end
end

-- things to do when the module is disabled
function Prat_Substitutions:OnModuleDisable()
	-- unregister events
	 Prat:UnregisterAllPatterns(self.name)
end

--[[------------------------------------------------
    Core Functions
------------------------------------------------]]--

local function Zone(...)
    return Prat:RegisterMatch(GetZoneText(), "OUTBOUND")
end
local function Loc(...)
    return Prat:RegisterMatch(GetMinimapZoneText(), "OUTBOUND")
end
local function PlayerHP(...)
    return Prat:RegisterMatch(UnitHealth("player"), "OUTBOUND")
end
local function PlayerMaxHP(...)
    return Prat:RegisterMatch(UnitHealthMax("player"), "OUTBOUND")
end


Prat_Substitutions.modulePatterns = { 
    { pattern = "(%%zon)", matchfunc=Zone, type = "OUTBOUND"},
    { pattern = "(%%loc)", matchfunc=Loc, type = "OUTBOUND"},
    { pattern = "(%%hc)", matchfunc=PlayerHP, type = "OUTBOUND"},
    { pattern = "(%%hm)", matchfunc=PlayerMaxHP, type = "OUTBOUND"},
}


--[[
local function XXXX(...)
    return Prat:RegisterMatch(  , "OUTBOUND")
end
local function XXXX(...)
    return Prat:RegisterMatch(  , "OUTBOUND")
end
local function XXXX(...)
    return Prat:RegisterMatch(  , "OUTBOUND")
end
local function XXXX(...)
    return Prat:RegisterMatch(  , "OUTBOUND")
end
local function XXXX(...)
    return Prat:RegisterMatch(  , "OUTBOUND")
end
local function XXXX(...)
    return Prat:RegisterMatch(  , "OUTBOUND")
end
local function XXXX(...)
    return Prat:RegisterMatch(  , "OUTBOUND")
end
local function XXXX(...)
    return Prat:RegisterMatch(  , "OUTBOUND")
end

			-- Player substitutions
			-- %hc (current health)
			if string.find(text, "(%%hc)") then
				text = string.gsub(text, "(%%hc)", UnitHealth("player"))
			end    
			-- %hm (max health)
			if string.find(text, "(%%hm)") then
				text = string.gsub(text,"(%%hm)", UnitHealthMax("player"))
			end    
			-- %hp (percentage health)
			if string.find(text, "(%%hp)") then
				text = string.gsub(text, "(%%hp)", string.format("%.0f%%%%", floor(100 * (UnitHealth("player") / UnitHealthMax("player")))))
			end    
			-- %mc (current mana)
			if string.find(text, "(%%mc)") then
				text = string.gsub(text, "(%%mc)", UnitMana("player"))
			end    
			-- %mm (max mana)
			if string.find(text, "(%%mm)") then
				text = string.gsub(text, "(%%mm)", UnitManaMax("player"))
			end    
			-- %mp (percentage mana)
			if string.find(text, "(%%mp)") then
				text = string.gsub(text,"(%%mp)", string.format("%.0f%%%%", floor(100 * (UnitMana("player")/UnitManaMax("player")))))
			end    
			
			-- Target substitutions
			if string.find(text, "(%%t)") then
				-- %thp (target's percentage health)
				if string.find(text, "(%%thp)") then
					local str = "<no target>"
					if UnitExists("target") then
						str = string.format("%.0f%%%%", floor(100 * (UnitHealth("target") / UnitHealthMax("target"))))
					end
					text = string.gsub(text, "(%%thp)", str)
				end    
				-- %tc (class of target)
				if string.find(text, "(%%tc)") then
					local class = "<no target>"
					if UnitExists("target") then
						class = UnitClass("target")
					end
					text = string.gsub(text,"(%%tc)", class)
				end
				-- %tr (race)
				if string.find(text,"(%%tr)") then
					local race = "<no target>"
					if UnitExists("target") then
						if UnitIsPlayer("target") then
							race = UnitRace("target")
						else
							race = UnitCreatureFamily("target")
							if not race then
								race = UnitCreatureType("target")
							end	
						end
					end
					text = string.gsub(text,"(%%tr)", race)
				end
				-- %ts (sex)
				if string.find(text, "(%%ts)") then
					local sex = "<no target>"
					if UnitExists("target") then
						s = UnitSex("target") 
						if(s == 2) then
							sex = "male"
						elseif (s == 3) then
							sex = "female"
						else
							sex = "unknown sex"
						end
					end
					text = string.gsub(text,"(%%ts)", sex)
				end
				-- %tl (level)
				if string.find(text, "(%%tl)") then
					local level = "<no target>"
					if UnitExists("target") then
						level = UnitLevel("target")
					end
					text = string.gsub(text,"(%%tl)", level)
				end
				-- %tvr (PVP rank)
				if string.find(text, "(%%tvr)") then
					local rank, name
					if not UnitExists("target") then
						rank = "<no target>"
					else
						if UnitIsPlayer("target") then
							_,_,rank,name = string.find(UnitPVPName("target"), "(.+) (.+)")
						end
						if not rank then
							rank = "<no rank>"
						end
					end
					text = string.gsub(text,"(%%tvr)", rank)
				end
				-- %tvn (PVP name)
				if string.find(text, "(%%tvn)") then
					local name = "<no target>"
					if UnitExists("target") then
						name = UnitPVPName("target")
					end
					text = string.gsub(text,"(%%tvn)", name)
				end
				-- %tps (possesive)
				if string.find(text,"(%%tps)") then
					local p = "<no target>"
					if UnitExists("target") then
						local s = UnitSex("target") 
						if(s == 2) then
							p = "his"
						elseif (s == 3) then
							p = "hers"
						else
							p = "its"
						end
					end
					text = string.gsub(text,"(%%tps)", p)
				end
				-- %tpn (pronoun)
				if string.find(text,"(%%tpn)") then
					local p = "<no target>"
					if UnitExists("target") then
						local s = UnitSex("target") 
						if(s == 2) then
							p = "him"
						elseif (s == 3) then
							p = "her"
						else
							p = "it"
						end
					end
					text = string.gsub(text,"(%%tpn)", p)
				end
			end
			
			-- Focus substitutions
			if string.find(text, "(%%f)") then
				-- %fhp (focus' percentage health)
				if string.find(text, "(%%fhp)") then
					local str = "<no focus>"
					if UnitExists("focus") then
						str = string.format("%.0f%%%%", floor(100 * (UnitHealth("focus") / UnitHealthMax("focus"))))
					end
					text = string.gsub(text, "(%%fhp)", str)
				end    
				-- %fr (race of focus)
				if string.find(text, "(%%fr)") then
					local race = "<no focus>"
					if UnitExists("focus") then
						if UnitIsPlayer("focus") then
							race = UnitRace("focus")
						else
							race = UnitCreatureFamily("focus")
							if(race == nil) then
								race = UnitCreatureType("focus")
							end	
						end
					end
					text = string.gsub(text,"(%%fr)", race)
				end
				-- %fc (class of focus)
				if string.find(text, "(%%fc)") then
					local class = "<no focus>"
					if UnitExists("focus") then
						class = UnitClass("focus")
					end
					text = string.gsub(text,"(%%fc)", class)
				end
				-- %fs (sex of focus)
				if string.find(text, "(%%fs)") then
					local sex = "<no focus>"
					if UnitExists("focus") then
						s = UnitSex("focus") 
						if(s == 2) then
							sex = "male"
						elseif (s == 3) then
							sex = "female"
						else
							sex = "unknown sex"
						end
					end
					text = string.gsub(text,"(%%fs)", sex)
				end
				-- %fl (level of focus)
				if string.find(text, "(%%fl)") then
					local level = "<no focus>"
					if UnitExists("focus") then
						level = UnitLevel("focus")
					end
					text = string.gsub(text,"(%%fl)", level)
				end
				-- %fvr (PVP rank)
				if string.find(text, "(%%fvr)") then
					local rank, name
					if not UnitExists("focus") then
						rank = "<no focus>"
					else
						if UnitIsPlayer("focus") then
							_,_,rank,name = string.find(UnitPVPName("focus"), "(.+) (.+)")
						end
						if not rank then
							rank = "<no rank>"
						end
					end
					text = string.gsub(text,"(%%fvr)", rank)
				end
				-- %fvn (PVP name of focus)
				if string.find(text, "(%%fvn)") then
					local name = "<no focus>"
					if UnitExists("focus") then
						name = UnitPVPName("focus")
					end
					text = string.gsub(text,"(%%fvn)", name)
				end
				-- %fps (possesive of focus)
				if string.find(text,"(%%fps)") then
					local p = "<no focus>"
					if UnitExists("focus") then
						local s = UnitSex("focus") 
						if(s == 2) then
							p = "his"
						elseif (s == 3) then
							p = "hers"
						else
							p = "its"
						end
					end
					text = string.gsub(text,"(%%fps)", p)
				end
				-- %fpn (pronoun of focus)
				if string.find(text,"(%%fpn)") then
					local p = "<no focus>"
					if UnitExists("focus") then
						local s = UnitSex("focus") 
						if(s == 2) then
							p = "him"
						elseif (s == 3) then
							p = "her"
						else
							p = "it"
						end
					end
					text = string.gsub(text,"(%%fpn)", p)
				end
				-- %f (name of focus)
				if string.find(text,"(%%f)") then
					local name = "<no focus>"
					if UnitExists("focus") then
						name = UnitName("focus")
					end
					text = string.gsub(text,"(%%f)", name)
				end
			end		

			-- Other substitutions
			-- %tt (target of target)
			if string.find(text, "(%%tt)") then
				local targetoftarget = UnitName("targettarget")
				if(targetoftarget == nil) then
					targetoftarget="<no target>"
				end
				text = string.gsub(text,"(%%tt)", targetoftarget)
			end    

			-- %zon (zone)
			if string.find(text, "(%%zon)") then
				text = string.gsub(text,"(%%zon)", GetZoneText())
			end
		
			-- %loc (minimap location)
			if string.find(text, "(%%loc)") then
				text = string.gsub(text,"(%%loc)", GetMinimapZoneText())
			end 
		
			-- %pos
			if string.find(text, "(%%pos)") then
				local x,y = GetPlayerMapPosition("player")
				local str = "("..math.floor((x * 100) + 0.5)..","..math.floor((y * 100) + 0.5)..")"
				text = string.gsub(text,"(%%pos)", str)
			end
			
--]]
